from pytube import YouTube
from PrtSc.PrtSc import PrtSc
from OS_Platform.OS_Platform import getOSPlatform
import psutil
import matplotlib.pyplot as plt
import matplotlib
from batterycharge.batterycharge import getBatteryCharge
import tkinter as tk
import tkinter.ttk as ttk
import webbrowser
from cv2 import imshow,VideoCapture,waitKey,destroyAllWindows
from pyjokes import get_joke
import flask
import rotatescreen
import pyautogui
import sqlite3
import pyttsx3
import requests
import json
from HCF import HCF
class Joke:
    def __init__(self):
        self.joke=get_joke()

    def print(self):
        print(self.joke)

    def say(self,accent):
        x=TextToSpeechInit()
        x.setAccent(accent)
        x.speak(self.joke)

class openWebcam:
    def __init__(self,codeorkey):
        cam=VideoCapture(0)
        while True:
            check,frame=cam.read()
            imshow("Frame",frame)
            key=waitKey(1)
            if key==codeorkey:
                break

        cam.release()
        destroyAllWindows()


class YoutubeVideo:
    def __init__(self,url):
        self.url=url

    def download(self):
        o=YouTube(self.url)
        vid=o.streams.get_by_itag(22)
        vid.download()

    def openVideo(self):
        webbrowser.open(self.url)


class getLocation:
    def __init__(self):
        self.details=requests.get('https://ipinfo.io/')
        self.locationdata=self.details.json()

class DateError(BaseException):
    def __init__(self):
        print("Invalid Date Given.")

def dayOnDate(d,m,y):
    disallowed=[
        d>31,
        m>12
    ]
    if any(disallowed):
        raise DateError

    monthcodes=[0,3,3,6,1,4,6,2,5,0,3,5]
    rem=(y//100)%4
    ccode=0
    if rem==1:
        ccode=5
    elif rem==2:
        ccode=3
    elif rem==3:
        ccode=1

    leap=0
    if y%4==0 and y%100!=0 and m>2:
        leap=1
    elif y%400==0 and m>2:
        leap=1
    else:
        pass
    daycode=(d+monthcodes[m-1]+(y%100)-1+((y%100)-1)//4+ccode+leap)%7
    days=['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday']
    return days[daycode]

class AccentError(BaseException):
    def __init__(self):
        print("invalid Accent Name... Exiting...")

class TextToSpeechInit:
    def __init__(self):
        self.speaker=pyttsx3.init()
        self.accent='male'
        self.voices=self.speaker.getProperty('voices')
        self.speaker.setProperty('voice', self.voices[0].id)

    def setAccent(self,accent):
        self.accent=accent.lower()
        if self.accent!='male' and self.accent!='female':
            raise AccentError
        if self.accent=='female':
            self.speaker.setProperty('voice',self.voices[1].id)
        else:
            self.speaker.setProperty('voice', self.voices[0].id)

    def speak(self,msg):
        self.speaker.say(msg)
        self.speaker.runAndWait()







#!/usr/bin/env python3
import tkinter as tk
import tkinter.ttk as ttk
import tkinter.messagebox as msg

import yaml

from tkinter import filedialog

from textarea import TextArea
from linenumbers import LineNumbers
from highlighter import Highlighter
from findwindow import FindWindow
from colorchooser import colorChooser


class TextEditor(tk.Tk):
    def __init__(self,defaultyaml,langyaml):
        super().__init__()

        self.title('CodeWithPy 2021 v1')
        self.geometry('800x600')
        self.foreground = 'black'
        self.background = 'lightgrey'
        self.text_foreground = 'black'
        self.text_background='white'

        self.load_scheme_file(defaultyaml)
        self.configure_ttk_elements()

        self.font_size = 20
        self.font_family = "Ubuntu Mono"

        self.text_area = TextArea(self, bg=self.text_background, fg=self.text_foreground, undo=True,
                                  font=(self.font_family, self.font_size))

        self.scrollbar = ttk.Scrollbar(orient="vertical", command=self.scroll_text)
        self.text_area.configure(yscrollcommand=self.scrollbar.set)

        self.line_numbers = LineNumbers(self, self.text_area, bg="grey", fg="white", width=1,font=("Ubuntu Mono",20))
        self.highlighter = Highlighter(self.text_area, langyaml)

        self.menu = tk.Menu(self, bg=self.background, fg=self.foreground)
        self.all_menus = [self.menu]

        sub_menu_items = ["file", "edit", "tools", "help"]
        self.generate_sub_menus(sub_menu_items)
        self.configure(menu=self.menu)

        self.right_click_menu = tk.Menu(self, bg=self.background, fg=self.foreground, tearoff=0)
        self.right_click_menu.add_command(label='Cut', command=self.edit_cut)
        self.right_click_menu.add_command(label='Copy', command=self.edit_copy)
        self.right_click_menu.add_command(label='Paste', command=self.edit_paste)
        self.all_menus.append(self.right_click_menu)

        self.scrollbar.pack(side=tk.RIGHT, fill=tk.Y)
        self.line_numbers.pack(side=tk.LEFT, fill=tk.Y)
        self.text_area.pack(side=tk.LEFT, fill=tk.BOTH, expand=1)

        self.bind_events()

        self.open_file = ''

        self.mainloop()

    def bind_events(self):
        self.text_area.bind("<MouseWheel>", self.scroll_text)
        self.text_area.bind("<Button-4>", self.scroll_text)
        self.text_area.bind("<Button-5>", self.scroll_text)
        self.text_area.bind("<Button-3>", self.show_right_click_menu)

        self.bind('<Control-f>', self.show_find_window)

        self.bind('<Control-n>', self.file_new)
        self.bind('<Control-o>', self.file_open)
        self.bind('<Control-s>', self.file_save)

        self.bind('<Control-h>', self.help_about)

        self.bind('<Control-m>', self.tools_change_syntax_highlighting)
        self.bind('<Control-g>', self.tools_change_color_scheme)

        self.line_numbers.bind("<MouseWheel>", lambda e: "break")
        self.line_numbers.bind("<Button-4>", lambda e: "break")
        self.line_numbers.bind("<Button-5>", lambda e: "break")

    def scroll_text(self, *args):
        if len(args) > 1:
            self.text_area.yview_moveto(args[1])
            self.line_numbers.yview_moveto(args[1])
        else:
            event = args[0]
            if event.delta:
                move = -1 * (event.delta / 120)
            else:
                if event.num == 5:
                    move = 1
                else:
                    move = -1

            self.text_area.yview_scroll(int(move), "units")
            self.line_numbers.yview_scroll(int(move) * 3, "units")

    def show_find_window(self, event=None):
        FindWindow(self.text_area)

    def show_right_click_menu(self, event):
        x = self.winfo_x() + self.text_area.winfo_x() + event.x
        y = self.winfo_y() + self.text_area.winfo_y() + event.y
        self.right_click_menu.post(x, y)

    def generate_sub_menus(self, sub_menu_items):
        window_methods = [method_name for method_name in dir(self)
                          if callable(getattr(self, method_name))]
        tkinter_methods = [method_name for method_name in dir(tk.Tk)
                           if callable(getattr(tk.Tk, method_name))]

        my_methods = [method for method in set(window_methods) - set(tkinter_methods)]
        my_methods = sorted(my_methods)

        for item in sub_menu_items:
            sub_menu = tk.Menu(self.menu, tearoff=0, bg=self.background, fg=self.foreground)
            matching_methods = []
            for method in my_methods:
                if method.startswith(item):
                    matching_methods.append(method)

            for match in matching_methods:
                actual_method = getattr(self, match)
                method_shortcut = actual_method.__doc__.strip()
                friendly_name = ' '.join(match.split('_')[1:])
                sub_menu.add_command(label=friendly_name.title(), command=actual_method, accelerator=method_shortcut)

            self.menu.add_cascade(label=item.title(), menu=sub_menu)
            self.all_menus.append(sub_menu)

    def show_about_page(self):
        msg.showinfo("About", "Python Code Editor")

    def load_syntax_highlighting_file(self):
        syntax_file = filedialog.askopenfilename(filetypes=[("YAML file", ("*.yaml", "*.yml"))])
        if syntax_file:
            self.highlighter.clear_highlight()
            self.highlighter = Highlighter(self.text_area, syntax_file)
            self.highlighter.force_highlight()

    def load_scheme_file(self, scheme):
        with open(scheme, 'r') as stream:
            try:
                config = yaml.load(stream=stream,Loader=yaml.FullLoader)
            except yaml.YAMLError as error:
                print(error)
                return

        self.foreground = config['foreground']
        self.background = config['background']
        self.text_foreground = config['text_foreground']
        self.text_background = config['text_background']

    def change_color_scheme(self):
        colorChooser(self)

    def apply_color_scheme(self, foreground, background, text_foreground, text_background):
        self.text_area.configure(fg=text_foreground, bg=text_background)
        self.background = background
        self.foreground = foreground
        for menu in self.all_menus:
            menu.configure(bg=self.background, fg=self.foreground)
        self.configure_ttk_elements()

    def configure_ttk_elements(self):
        style = ttk.Style()
        style.configure('editor.TLabel', foreground=self.foreground, background=self.background)
        style.configure('editor.TButton', foreground=self.foreground, background=self.background)

    # =========== Menu Functions ==============

    def file_new(self, event=None):
        """
        Ctrl+N
        """
        self.text_area.delete(1.0, tk.END)
        self.open_file = None
        self.line_numbers.force_update()

    def file_open(self, event=None):
        """
        Ctrl+O
        """
        file_to_open = filedialog.askopenfilename()
        if file_to_open:
            self.open_file = file_to_open

            self.text_area.display_file_contents(file_to_open)
            self.highlighter.force_highlight()
            self.line_numbers.force_update()

    def file_save(self, event=None):
        """
        Ctrl+S
        """
        current_file = self.open_file if self.open_file else None
        if not current_file:
            current_file = filedialog.asksaveasfilename()

        if current_file:
            contents = self.text_area.get(1.0, tk.END)
            with open(current_file, 'w') as file:
                file.write(contents)

    def edit_cut(self, event=None):
        """
        Ctrl+X
        """
        self.text_area.event_generate("<Control-x>")
        self.line_numbers.force_update()

    def edit_paste(self, event=None):
        """
        Ctrl+V
        """
        self.text_area.event_generate("<Control-v>")
        self.line_numbers.force_update()
        self.highlighter.force_highlight()

    def edit_copy(self, event=None):
        """
        Ctrl+C
        """
        self.text_area.event_generate("<Control-c>")

    def edit_select_all(self, event=None):
        """
        Ctrl+A
        """
        self.text_area.event_generate("<Control-a>")

    def edit_find_and_replace(self, event=None):
        """
        Ctrl+F
        """
        self.show_find_window()

    def help_about(self, event=None):
        """
        Ctrl+H
        """
        self.show_about_page()

    def tools_change_syntax_highlighting(self, event=None):
        """
        Ctrl+M
        """
        self.load_syntax_highlighting_file()

    def tools_change_color_scheme(self, event=None):
        """
        Ctrl+G
        """
        self.change_color_scheme()


#!/usr/bin/env python3
import tkinter as tk
import tkinter.ttk as ttk


class FindWindow(tk.Toplevel):
    def __init__(self, master, **kwargs):
        super().__init__(**kwargs)

        self.master = master

        self.geometry('350x100')
        self.title('Find and Replace')
        self.transient(self.master)
        self.configure(bg=self.master.master.background)

        self.text_to_find = tk.StringVar()
        self.text_to_replace_with = tk.StringVar()

        top_frame = tk.Frame(self, bg=self.master.master.background)
        middle_frame = tk.Frame(self, bg=self.master.master.background)
        bottom_frame = tk.Frame(self, bg=self.master.master.background)

        find_entry_label = ttk.Label(top_frame, text="Find: ", style="editor.TLabel")
        self.find_entry = ttk.Entry(top_frame, textvar=self.text_to_find)

        replace_entry_label = ttk.Label(middle_frame, text="Replace: ", style="editor.TLabel")
        self.replace_entry = ttk.Entry(middle_frame, textvar=self.text_to_replace_with)

        self.find_button = ttk.Button(bottom_frame, text="Find", command=self.on_find, style="editor.TButton")
        self.replace_button = ttk.Button(bottom_frame, text="Replace", command=self.on_replace, style="editor.TButton")
        self.cancel_button = ttk.Button(bottom_frame, text="Cancel", command=self.on_cancel, style="editor.TButton")

        find_entry_label.pack(side=tk.LEFT, padx=(20, 0))
        self.find_entry.pack(side=tk.LEFT, fill=tk.X, expand=1)

        replace_entry_label.pack(side=tk.LEFT)
        self.replace_entry.pack(side=tk.LEFT, fill=tk.X, expand=1)

        self.find_button.pack(side=tk.LEFT, padx=(85, 0))
        self.replace_button.pack(side=tk.LEFT, padx=(20, 20))
        self.cancel_button.pack(side=tk.RIGHT, padx=(0, 30))

        top_frame.pack(side=tk.TOP, expand=1, fill=tk.X, padx=30)
        middle_frame.pack(side=tk.TOP, expand=1, fill=tk.X, padx=30)
        bottom_frame.pack(side=tk.TOP, expand=1, fill=tk.X)

        self.find_entry.focus_force()

        self.protocol("WM_DELETE_WINDOW", self.on_cancel)

    def on_find(self):
        self.master.find(self.text_to_find.get())

    def on_replace(self):
        self.master.replace_text(self.text_to_find.get(), self.text_to_replace_with.get())

    def on_cancel(self):
        self.master.cancel_find()
        self.destroy()


if __name__ == '__main__':
    mw = tk.Tk()
    fw = FindWindow(mw)
    mw.mainloop()



#!/usr/bin/env python3
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.colorchooser import askcolor


class colorChooser(tk.Toplevel):
    def __init__(self, master, **kwargs):
        super().__init__(**kwargs)
        self.master = master

        self.transient(self.master)
        self.geometry('400x300')
        self.title('color Scheme')
        self.configure(bg=self.master.background)

        self.chosen_background_color = tk.StringVar()
        self.chosen_foreground_color = tk.StringVar()
        self.chosen_text_background_color = tk.StringVar()
        self.chosen_text_foreground_color = tk.StringVar()

        self.chosen_background_color.set(self.master.background)
        self.chosen_foreground_color.set(self.master.foreground)
        self.chosen_text_background_color.set(self.master.text_background)
        self.chosen_text_foreground_color.set(self.master.text_foreground)

        window_frame = tk.Frame(self, bg=self.master.background)
        window_foreground_frame = tk.Frame(window_frame, bg=self.master.background)
        window_background_frame = tk.Frame(window_frame, bg=self.master.background)

        text_frame = tk.Frame(self, bg=self.master.background)
        text_foreground_frame = tk.Frame(text_frame, bg=self.master.background)
        text_background_frame = tk.Frame(text_frame, bg=self.master.background)

        self.all_frames = [window_frame, window_foreground_frame, window_background_frame,
                           text_frame, text_foreground_frame, text_background_frame]

        window_label = ttk.Label(window_frame, text="Window:", anchor=tk.W, style="editor.TLabel")
        foreground_label = ttk.Label(window_foreground_frame, text="Foreground:", anchor=tk.E, style="editor.TLabel")
        background_label = ttk.Label(window_background_frame, text="Background:", anchor=tk.E, style="editor.TLabel")

        text_label = ttk.Label(text_frame, text="Editor:", anchor=tk.W, style="editor.TLabel")
        text_foreground_label = ttk.Label(text_foreground_frame, text="Foreground:", anchor=tk.E, style="editor.TLabel")
        text_background_label = ttk.Label(text_background_frame, text="Background:", anchor=tk.E, style="editor.TLabel")

        foreground_color_chooser = ttk.Button(window_foreground_frame, text="Change Foreground color", width=26, style="editor.TButton",
                                               command=lambda sv=self.chosen_foreground_color: self.set_color(sv))
        background_color_chooser = ttk.Button(window_background_frame, text="Change Background color", width=26,
                                               style="editor.TButton",
                                               command=lambda sv=self.chosen_background_color: self.set_color(sv))
        text_foreground_color_chooser = ttk.Button(text_foreground_frame, text="Change Text Foreground color",
                                                    width=26, style="editor.TButton",
                                                    command=lambda sv=self.chosen_text_foreground_color: self.set_color(sv))
        text_background_color_chooser = ttk.Button(text_background_frame, text="Change Text Background color",
                                                    width=26, style="editor.TButton",
                                                    command=lambda sv=self.chosen_text_background_color: self.set_color(sv))

        foreground_color_preview = ttk.Label(window_foreground_frame, textvar=self.chosen_foreground_color,
                                              style="editor.TLabel")
        background_color_preview = ttk.Label(window_background_frame, textvar=self.chosen_background_color,
                                              style="editor.TLabel")
        text_foreground_color_preview = ttk.Label(text_foreground_frame, textvar=self.chosen_text_foreground_color,
                                                   style="editor.TLabel")
        text_background_color_preview = ttk.Label(text_background_frame, textvar=self.chosen_text_background_color,
                                                   style="editor.TLabel")


        window_frame.pack(side=tk.TOP, fill=tk.X, expand=1)
        window_label.pack(side=tk.TOP, fill=tk.X)

        window_foreground_frame.pack(side=tk.TOP, fill=tk.X, expand=1)
        window_background_frame.pack(side=tk.TOP, fill=tk.X, expand=1)

        foreground_label.pack(side=tk.LEFT, padx=30, pady=10)
        foreground_color_chooser.pack(side=tk.LEFT)
        foreground_color_preview.pack(side=tk.LEFT, expand=1, fill=tk.X, padx=(15, 0))

        background_label.pack(side=tk.LEFT, fill=tk.X, padx=(30, 27))
        background_color_chooser.pack(side=tk.LEFT)
        background_color_preview.pack(side=tk.LEFT, expand=1, fill=tk.X, padx=(15, 0))

        text_frame.pack(side=tk.TOP, fill=tk.X, expand=1)
        text_label.pack(side=tk.TOP, fill=tk.X)

        text_foreground_frame.pack(side=tk.TOP, fill=tk.X, expand=1)
        text_background_frame.pack(side=tk.TOP, fill=tk.X, expand=1)

        text_foreground_label.pack(side=tk.LEFT, padx=30, pady=10)
        text_foreground_color_chooser.pack(side=tk.LEFT)
        text_foreground_color_preview.pack(side=tk.LEFT, expand=1, fill=tk.X, padx=(15, 0))

        text_background_label.pack(side=tk.LEFT, fill=tk.X, padx=(30, 27))
        text_background_color_chooser.pack(side=tk.LEFT)
        text_background_color_preview.pack(side=tk.LEFT, expand=1, fill=tk.X, padx=(15, 0))

        save_button = ttk.Button(self, text="save", command=self.save, style="editor.TButton")
        save_button.pack(side=tk.BOTTOM, pady=(0, 20))


    def set_color(self, sv):
        choice = askcolor()[1]
        sv.set(choice)

    def save(self):
        yaml_file_contents = f"background: '{self.chosen_background_color.get()}'\n" \
                             + f"foreground: '{self.chosen_foreground_color.get()}'\n" \
                             + f"text_background: '{self.chosen_text_background_color.get()}'\n" \
                             + f"text_foreground: '{self.chosen_text_foreground_color.get()}'\n"

        with open("schemes/default.yaml", "w") as yaml_file:
            yaml_file.write(yaml_file_contents)

        self.master.apply_color_scheme(self.chosen_foreground_color.get(), self.chosen_background_color.get(),
                                        self.chosen_text_foreground_color.get(), self.chosen_text_background_color.get())
        for frame in self.all_frames:
            frame.configure(bg=self.chosen_background_color.get())

        self.configure(bg=self.chosen_background_color.get())






#!/usr/bin/env python3
import tkinter as tk
import tkinter.messagebox as msg

class TextArea(tk.Text):
    def __init__(self, master, **kwargs):
        super().__init__(**kwargs)

        self.master = master

        self.config(wrap=tk.WORD)  # CHAR NONE

        self.tag_configure('find_match', background="yellow")
        self.find_match_index = None
        self.find_search_starting_index = 1.0

        self.bind_events()

    def bind_events(self):
        self.bind('<Control-a>', self.select_all)
        self.bind('<Control-c>', self.copy)
        self.bind('<Control-v>', self.paste)
        self.bind('<Control-x>', self.cut)
        self.bind('<Control-y>', self.redo)
        self.bind('<Control-z>', self.undo)

    def cut(self, event=None):
        self.event_generate("<<Cut>>")

        return "break"

    def copy(self, event=None):
        self.event_generate("<<Copy>>")

        return "break"

    def paste(self, event=None):
        self.event_generate("<<Paste>>")

        return "break"

    def undo(self, event=None):
        self.event_generate("<<Undo>>")

        return "break"

    def redo(self, event=None):
        self.event_generate("<<Redo>>")

        return "break"

    def select_all(self, event=None):
        self.tag_add("sel", 1.0, tk.END)

        return "break"

    def find(self, text_to_find):
        length = tk.IntVar()
        idx = self.search(text_to_find, self.find_search_starting_index, stopindex=tk.END, count=length)

        if idx:
            self.tag_remove('find_match', 1.0, tk.END)

            end = f'{idx}+{length.get()}c'
            self.tag_add('find_match', idx, end)
            self.see(idx)

            self.find_search_starting_index = end
            self.find_match_index = idx
        else:
            if self.find_match_index != 1.0:
                if msg.askyesno("No more results", "No further matches. Repeat from the beginning?"):
                    self.find_search_starting_index = 1.0
                    self.find_match_index = None
                    return self.find(text_to_find)
            else:
                msg.showinfo("No Matches", "No matching text found")

    def replace_text(self, target, replacement):
        if self.find_match_index:
            current_found_index_line = str(self.find_match_index).split('.')[0]

            end = f"{self.find_match_index}+{len(target)}c"
            self.replace(self.find_match_index, end, replacement)

            self.find_search_starting_index = current_found_index_line + '.0'

    def cancel_find(self):
        self.find_search_starting_index = 1.0
        self.find_match_index = None
        self.tag_remove('find_match', 1.0, tk.END)

    def display_file_contents(self, filepath):
        with open(filepath, 'r') as file:
            self.delete(1.0, tk.END)
            self.insert(1.0, file.read())




from setuptools import setup
setup(
    name="MNSD",
    version="2021.0.3",
    author="Pranav",
    packages=["MNSD"],
    license="MIT",
    install_requires=['matplotlib','pytube','pyyaml','PrtSc','OS_Platform','psutil','batterycharge','opencv-python','pyjokes','requests','flask','pyautogui','pypiwin32','rotate-screen'],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
    py_requires='>3.7',
)


#!/usr/bin/env python3
import tkinter as tk


class LineNumbers(tk.Text):
    def __init__(self, master, text_widget, **kwargs):
        super().__init__(master, **kwargs)
        self.text_widget = text_widget
        self.text_widget.bind('<KeyPress>', self.on_key_press)

        self.insert(1.0, '1')
        self.configure(state='disabled')

    def on_key_press(self, event=None):
        final_index = str(self.text_widget.index(tk.END))
        num_of_lines = final_index.split('.')[0]
        line_numbers_string = "\n".join(str(no + 1) for no in range(int(num_of_lines)))
        width = len(str(num_of_lines))

        self.configure(state='normal', width=width)
        self.delete(1.0, tk.END)
        self.insert(1.0, line_numbers_string)
        self.configure(state='disabled')

    def force_update(self):
        self.on_key_press()




#!/usr/bin/env python3
import tkinter as tk
import yaml

class Highlighter:
    def __init__(self, text_widget, syntax_file):
        self.text_widget = text_widget
        self.syntax_file = syntax_file
        self.categories = None
        self.numbers_color = "blue"
        self.strings_color = "red"

        self.disallowed_previous_chars = ["_", "-", "."]

        self.parse_syntax_file()

        self.text_widget.bind('<KeyRelease>', self.on_key_release)

    def on_key_release(self, event=None):
        self.highlight()

    def parse_syntax_file(self):
        with open(self.syntax_file, 'r') as stream:
            try:
                config = yaml.load(stream=stream,Loader=yaml.FullLoader)
            except yaml.YAMLError as error:
                print(error)
                return

        self.categories = config['categories']
        self.numbers_color = config['numbers']['color']
        self.strings_color = config['strings']['color']

        self.configure_tags()

    def configure_tags(self):
        for category in self.categories.keys():
            color = self.categories[category]['color']
            self.text_widget.tag_configure(category, foreground=color)

        self.text_widget.tag_configure("number", foreground=self.numbers_color)
        self.text_widget.tag_configure("string", foreground=self.strings_color)

    def highlight(self, event=None):
        length = tk.IntVar()
        for category in self.categories:
            matches = self.categories[category]['matches']
            for keyword in matches:
                start = 1.0
                keyword = keyword + "[^A-Za-z_-]"
                idx = self.text_widget.search(keyword, start, stopindex=tk.END, count=length, regexp=1)
                while idx:
                    char_match_found = int(str(idx).split('.')[1])
                    line_match_found = int(str(idx).split('.')[0])
                    if char_match_found > 0:
                        previous_char_index = str(line_match_found) + '.' + str(char_match_found - 1)
                        previous_char = self.text_widget.get(previous_char_index, previous_char_index + "+1c")

                        if previous_char.isalnum() or previous_char in self.disallowed_previous_chars:
                            end = f"{idx}+{length.get() - 1}c"
                            start = end
                            idx = self.text_widget.search(keyword, start, stopindex=tk.END, regexp=1)
                        else:
                            end = f"{idx}+{length.get() - 1}c"
                            self.text_widget.tag_add(category, idx, end)

                            start = end
                            idx = self.text_widget.search(keyword, start, stopindex=tk.END, regexp=1)
                    else:
                        end = f"{idx}+{length.get() - 1}c"
                        self.text_widget.tag_add(category, idx, end)

                        start = end
                        idx = self.text_widget.search(keyword, start, stopindex=tk.END, regexp=1)

        self.highlight_regex(r"(\d)+[.]?(\d)*", "number")
        self.highlight_regex(r"[\'][^\']*[\']", "string")
        self.highlight_regex(r"[\"][^\']*[\"]", "string")

    def highlight_regex(self, regex, tag):
        length = tk.IntVar()
        start = 1.0
        idx = self.text_widget.search(regex, start, stopindex=tk.END, regexp=1, count=length)
        while idx:
            end = f"{idx}+{length.get()}c"
            self.text_widget.tag_add(tag, idx, end)

            start = end
            idx = self.text_widget.search(regex, start, stopindex=tk.END, regexp=1, count=length)

    def force_highlight(self):
        self.highlight()

    def clear_highlight(self):
        for category in self.categories:
            self.text_widget.tag_remove(category, 1.0, tk.END)


if __name__ == '__main__':
    w = tk.Tk()
    h = Highlighter(tk.Text(w), 'languages/python.yaml')
    w.mainloop()



